#    Back In Time
#    Copyright (C) 2010 Ryan Ahearn
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os.path
import subprocess
import tempfile
import urwid
import gettext

import logger
import restorewidget

_=gettext.gettext

class SnapshotsWidget(urwid.WidgetWrap):

    def __init__( self, main_win, config, snapshots, path ):
        self._main_win = main_win
        self._config = config
        self._snapshots = snapshots
        self._path = path
        self._selected_left = ''
        self._selected_right = None

        left_side, right_side = self._build_snapshot_lists()
        pile_list = [
            urwid.Text( _('Diff versions of') + ': %s' % path ),
            urwid.Divider(),
            urwid.Columns( [
                urwid.Pile( left_side ),
                urwid.Pile( right_side )
                ], 1 ),
            urwid.Divider(),
            urwid.GridFlow( [
                urwid.AttrWrap( urwid.Button( _('Jump to'), self.jump ), 'button', 'button_focus' ),
                urwid.AttrWrap( urwid.Button( _('Diff'), self.diff ), 'button', 'button_focus' ),
                urwid.AttrWrap( urwid.Button( _('Cancel'), self.go_back ), 'button', 'button_focus' )
                ], 11, 2, 1, 'right' )
            ]

        display_widget = urwid.Pile( pile_list )
        urwid.WidgetWrap.__init__( self, display_widget )

    def _build_snapshot_lists( self ):
        snapshots_list = self._snapshots.get_snapshots_list()
        left_list = [
            urwid.Text( _('Snapshots') + ':' ),
            urwid.Divider()
            ]
        left_list_buttons = []
        right_list = [
            urwid.Text( _('Diff with') + ':' ),
            urwid.Divider()
            ]
        right_list_buttons = []
        
        path = self._snapshots.get_snapshot_path_to( self._main_win.selected_snapshot, self._path )
        isdir = os.path.isdir( path )

        # add now
        path = self._path
        if os.path.lexists( path ):
            if os.path.isdir( path ) == isdir:
                left_list.append( urwid.RadioButton( left_list_buttons, _('Now'), True, self.left_changed, '' ) )
                right_list.append( urwid.RadioButton( right_list_buttons, _('Now'), False, self.right_changed, '' ) )

        # add snapshots
        for snapshot in snapshots_list:
            path = self._snapshots.get_snapshot_path_to( snapshot, self._path )
            if os.path.lexists( path ):
                if os.path.isdir( path ) == isdir:
                    display_name = self._snapshots.get_snapshot_display_id( snapshot )
                    user_name = self._snapshots.get_snapshot_name( snapshot )
                    if len( user_name ) > 0:
                        display_name += ' - ' + user_name
                    if self._main_win.is_selected_snapshot( snapshot ):
                        selected = True
                        self._selected_right = snapshot
                    else:
                        selected = False
                    left_list.append( urwid.RadioButton( left_list_buttons, display_name, False, self.left_changed, snapshot ) )
                    right_list.append( urwid.RadioButton( right_list_buttons, display_name, selected, self.right_changed, snapshot ) )

        return left_list, right_list

    def left_changed( self, radio_button, new_state, user_data ):
        if new_state:
            self._selected_left = user_data

    def right_changed( self, radio_button, new_state, user_data ):
        if new_state:
            self._selected_right = user_data
                    
    def jump( self, button ):
        self._main_win.selected_snapshot = self._selected_left
        self._main_win.update_content( restorewidget.RestoreWidget( self._main_win, self._config ), 5 )

    def diff( self, button ):
        if self._selected_right is not None:
            self._main_win.update_content( DiffWidget( self._main_win, self._config, self._snapshots, self._path, self._selected_left, self._selected_right ), 5 )

    def go_back( self, button ):
        self._main_win.update_content( restorewidget.RestoreWidget( self._main_win, self._config ), 5 )

class DiffWidget(SnapshotsWidget):
    def __init__( self, main_win, config, snapshots, path, selected_left, selected_right ):
        self._main_win = main_win
        self._config = config

        path_left = snapshots.get_snapshot_path_to( selected_left, path )
        path_right = snapshots.get_snapshot_path_to( selected_right, path )

        diff_cmd = 'diff "%s" "%s"' % ( path_left, path_right )
        logger.info( 'diff cmd = "%s"' % diff_cmd )

        wc_cmd = '%s | wc -l' % diff_cmd
        callback = subprocess.Popen( wc_cmd, shell=True, stdout=subprocess.PIPE )
        output = callback.communicate()
        if main_win.main_loop.screen_size:
            rows = main_win.main_loop.screen_size[1]
        else:
            rows = main_win.main_loop.screen.get_cols_rows()[1]
        
        if int( output[0] ) > rows:
            temp_file, temp_file_name = tempfile.mkstemp( '_bitdiff', os.path.basename( path ), text=True )
            output_text = _('This diff was too long for the window, it has been written to') + ': %s' % temp_file_name
            callback = subprocess.Popen( diff_cmd, shell=True, stdout=temp_file )
            callback.communicate()
        else:
            callback = subprocess.Popen( diff_cmd, shell=True, stdout=subprocess.PIPE )
            output = callback.communicate()
            output_text = output[0]
            
        pile_list = [
            urwid.Text( diff_cmd ),
            urwid.Divider(),
            urwid.Text( output_text ),
            urwid.Divider(),
            urwid.AttrWrap( urwid.Button( _('OK'), self.go_back ), 'button', 'button_focus' )
            ]

        display_widget = urwid.Pile( pile_list )
        urwid.WidgetWrap.__init__( self, display_widget )
